// 
// 
// 

#include "CANopen.h"

void CANopenListener::handleReceivedMessage(CO_PDO *msg) // To be overridden by child class
{
}

void CANopenListener::handleReceivedMessage(CO_SDO *msg) // To be overridden by child class
{
}

void CANopen::printFrame(CAN_FRAME *frame, int mailbox) { //feil bruk av serial
	Serial.print("MB: ");
	if (mailbox > -1) Serial.print(mailbox);
	else Serial.print("???");
	Serial.print(" ID: 0x");
	Serial.print(frame->id, HEX);
	Serial.print(" Len: ");
	Serial.print(frame->length);
	Serial.print(" Data: 0x");
	for (int count = 0; count < frame->length; count++) {
		Serial.print(frame->data.bytes[count], HEX);
		Serial.print(" ");
	}
	Serial.print("\r\n");
}

void CANopen::gotFrame(CAN_FRAME * frame, int mailbox)
{
	COB_ID id = (COB_ID)frame->id;
	//printFrame(frame, mailbox);
	CO_PDO pdoFrame;
	CO_SDO sdoFrame;
	uint8_t scs;
	if (_listener != NULL) {
		switch (id)
		{
		case NMT:
			break;
		case SYNC:
			break;
		case Emergency:
			break;
		case TxPDO1:
		case TxPDO2:
		case TxPDO3:
		case TxPDO4:
			pdoFrame.id = id;
			pdoFrame.length = frame->length;
			pdoFrame.data.asInt = frame->data.value;
			_listener->handleReceivedMessage(&pdoFrame);
			break;
		case RxPDO1:
		case RxPDO2:
		case RxPDO3:
		case RxPDO4:
			break;
		case SDO_server2client:
			sdoFrame.index = ((frame->data.bytes[1] & 0x00ff) | (frame->data.bytes[2] << 8));
			sdoFrame.subIndex = frame->data.bytes[3];
			scs = frame->data.bytes[0] >> 5;
			if (scs == 2) {	// SDO initiate upload response
				sdoFrame.length = 4 - ((frame->data.bytes[0] >> 2) & 0x03);
				sdoFrame.data.asInt = frame->data.high;
			}
			else if (scs == 3) { // SDO initiate download response
				sdoFrame.length = 0;
				sdoFrame.data.asInt = 0;
			}
			_listener->handleReceivedMessage(&sdoFrame);
			break;
		case SDO_client2server:
			break;
		default:
			break;
		}
	}
}

bool CANopen::Init(uint32_t nodeId, uint32_t baudrate)
{
	_nodeId = nodeId;
	uint32_t ok = 0;
	switch (baudrate)
	{
	case 5000:
		ok = Can0.begin(CAN_BPS_5K);
		break;
	case 10000:
		ok = Can0.begin(CAN_BPS_10K);
		break;
	case 25000:
		ok = Can0.begin(CAN_BPS_25K);
		break;
	case 33333:
		ok = Can0.begin(CAN_BPS_33333);
		break;
	case 50000:
		ok = Can0.begin(CAN_BPS_50K);
		break;
	case 125000:
		ok = Can0.begin(CAN_BPS_125K);
		break;
	case 250000:
		ok = Can0.begin(CAN_BPS_250K);
		break;
	case 500000:
		ok = Can0.begin(CAN_BPS_500K);
		break;
	case 800000:
		ok = Can0.begin(CAN_BPS_800K);
		break;
	case 1000000:
		ok = Can0.begin(CAN_BPS_1000K);
		break;
	default:
		return false;
	}
	if (ok == 0) {
		return false; // Error initializing CAN module in Arduino
	}
	
	_outgoingCANframe.extended = false;
	_outgoingCANframe.priority = 10; // 0-15 lower is higher priority

	Can0.watchFor(COB_ID::TxPDO3);
	Can0.watchFor(COB_ID::TxPDO4);
	Can0.watchFor(COB_ID::SDO_server2client);
	Can0.attachObj(this);
	
	this->attachMBHandler(0);
	this->attachMBHandler(1);
	this->attachMBHandler(2);
	this->attachMBHandler(3);
	this->attachMBHandler(4);
	this->attachMBHandler(5);
	this->attachMBHandler(6);
	this->attachMBHandler(7);
	this->attachGeneralHandler();

	_state = NMTstate::Initialisation;
	return true;
}

void CANopen::AttachListener(CANopenListener *listener)
{
	_listener = listener;
}

bool CANopen::SDOdownload(CO_SDO msg)
{
	_outgoingCANframe.id = COB_ID::SDO_client2server;
	_outgoingCANframe.length = msg.length + 4;
	_outgoingCANframe.data.bytes[0] = (0x23 | ((0x04 - msg.length) << 2));	// Client command specifier, expedited transfer, (4 - length) bytes not containing data
	_outgoingCANframe.data.bytes[1] = (msg.index & 0xff);
	_outgoingCANframe.data.bytes[2] = (msg.index >> 8);
	_outgoingCANframe.data.bytes[3] = msg.subIndex;
	for (uint8_t i = 0; i < msg.length; i++)
	{
		_outgoingCANframe.data.bytes[i + 4] = msg.data.asBytes[i];
	}
	return Can0.sendFrame(_outgoingCANframe);
}

bool CANopen::SDOupload(CO_SDO msg)
{
	_outgoingCANframe.id = COB_ID::SDO_client2server;
	_outgoingCANframe.length = 8;
	_outgoingCANframe.data.bytes[0] = 0x40;	// Client command specifier, initiate upload request
	_outgoingCANframe.data.bytes[1] = (msg.index & 0xff);
	_outgoingCANframe.data.bytes[2] = (msg.index >> 8);
	_outgoingCANframe.data.bytes[3] = msg.subIndex;
	_outgoingCANframe.data.high = 0;
	return Can0.sendFrame(_outgoingCANframe);
}

bool CANopen::SendSynch()
{
	_outgoingCANframe.id = COB_ID::SYNC;
	_outgoingCANframe.length = 0x02;
	_outgoingCANframe.data.bytes[0] = 0;
	_outgoingCANframe.data.bytes[1] = 0;

	return Can0.sendFrame(_outgoingCANframe);
}

NMTstate CANopen::GetNMTstate()
{
	return _state;
}

// NMT commands
bool CANopen::StartRemoteNode()
{
	_outgoingCANframe.id = COB_ID::NMT;
	_outgoingCANframe.length = 0x02;
	_outgoingCANframe.data.bytes[0] = 0x01;
	_outgoingCANframe.data.bytes[1] = 0x00;

	if (Can0.sendFrame(_outgoingCANframe)) {
		_state = NMTstate::Operational;
		return true;
	}
	else
		return false;
}

bool CANopen::StopRemoteNode()
{
	_outgoingCANframe.id = COB_ID::NMT;
	_outgoingCANframe.length = 0x02;
	_outgoingCANframe.data.bytes[0] = 0x02;
	_outgoingCANframe.data.bytes[1] = 0x00;

	if (Can0.sendFrame(_outgoingCANframe)) {
		_state = NMTstate::Stopped;
		return true;
	}
	else
		return false;
}
bool CANopen::EnterPreOperational()
{
	_outgoingCANframe.id = COB_ID::NMT;
	_outgoingCANframe.length = 0x02;
	_outgoingCANframe.data.bytes[0] = 0x80;
	_outgoingCANframe.data.bytes[1] = 0x00;
	
	if (Can0.sendFrame(_outgoingCANframe)) {
		_state = NMTstate::PreOperational;
		return true;
	}
	else
		return false;
}

bool CANopen::ResetNode()
{
	_outgoingCANframe.id = COB_ID::NMT;
	_outgoingCANframe.length = 0x02;
	_outgoingCANframe.data.bytes[0] = 0x81;
	_outgoingCANframe.data.bytes[1] = 0x00;

	if (Can0.sendFrame(_outgoingCANframe)) {
		_state = NMTstate::PreOperational;
		return true;
	}
	else
		return false;
}

bool CANopen::ResetCommunication()
{
	_outgoingCANframe.id = COB_ID::NMT;
	_outgoingCANframe.length = 0x02;
	_outgoingCANframe.data.bytes[0] = 0x82;
	_outgoingCANframe.data.bytes[1] = 0x00;

	if (Can0.sendFrame(_outgoingCANframe)) {
		_state = NMTstate::PreOperational;
		return true;
	}
	else
		return false;
}
