// CANopen.h

#ifndef _CANOPEN_h
#define _CANOPEN_h

#if defined(ARDUINO) && ARDUINO >= 100
	#include "arduino.h"
#else
	#include "WProgram.h"
#endif

#include <due_can.h>

enum NMTstate {
	Initialisation,
	PreOperational,
	Operational,
	Stopped
};

enum COB_ID : uint32_t {
	NMT = 0x0,
	SYNC = 0x80,
	Emergency = 0x81,
	TxPDO1 = 0x181,
	RxPDO1 = 0x201,
	TxPDO2 = 0x281,
	RxPDO2 = 0x301,
	TxPDO3 = 0x381,
	RxPDO3 = 0x401,
	TxPDO4 = 0x481,
	RxPDO4 = 0x501,
	SDO_server2client = 0x581,
	SDO_client2server = 0x601
};

union Data32 {
	byte asBytes[4];
	int32_t asInt;
	uint32_t asUInt;
};

struct CO_SDO {
	uint8_t length;
	uint16_t index;
	uint8_t subIndex;
	Data32 data;
};

union Data64 {
	byte asBytes[8];
	uint64_t asInt;
};

struct CO_PDO {
	COB_ID id;
	uint8_t length;
	Data64 data;
};

class CANopenListener
{
public:
	virtual void handleReceivedMessage(CO_PDO *msg);
	virtual void handleReceivedMessage(CO_SDO *msg);
};

class CANopen : public CANListener
{
private:
	CAN_FRAME _outgoingCANframe;
	CAN_FRAME _incomingCANframe;
	uint8_t _nodeId;
	NMTstate _state;
	CANopenListener *_listener;

	void printFrame(CAN_FRAME *frame, int mailbox);
	void gotFrame(CAN_FRAME* frame, int mailbox);

public:
	bool Init(uint32_t nodeId, uint32_t baudrate);
	void AttachListener(CANopenListener *listener);
	bool SDOdownload(CO_SDO msg);
	bool SDOupload(CO_SDO msg);

	bool SendSynch();

	NMTstate GetNMTstate();

	// NMT commands
	bool StartRemoteNode();
	bool StopRemoteNode();
	bool EnterPreOperational();
	bool ResetNode();
	bool ResetCommunication();

};

#endif

