// 
// 
// 

#include "ExtendedADCShield.h"

void ExtendedADCShield::init(SemaphoreHandle_t SPImut, uint8_t number_bits, uint8_t CONVST, uint8_t BUSY, uint8_t RD)
{
	_SPImut = SPImut;
	_NUMBER_BITS = number_bits;
	_CONVST = CONVST;
	_BUSY = BUSY;
	_RD = RD;

	pinMode(_BUSY, INPUT);
	pinMode(_CONVST, OUTPUT);
	pinMode(_RD, OUTPUT);
	digitalWrite(_CONVST, LOW);
	digitalWrite(_RD, HIGH);

	_ADC_SPIsettings = SPISettings(ADC_SPI_FREQ, MSBFIRST, SPI_MODE0);
	SPI.begin();
	_lastPolarity = Polarity::UNIPOLAR;
	_lastRange = Range::RANGE5V;
}

float ExtendedADCShield::analogReadConfigNext(uint8_t channel, SignalType sgl_diff, Polarity uni_bipolar, Range range)
{
	uint8_t command = 0;
	uint16_t adc_code = 0;
	float voltage = 0;
	float sign = 1;

	command = buildCommand(channel, sgl_diff, uni_bipolar, range);
	adc_code = sendSetupGetData(command);
	
	if (_lastPolarity == Polarity::BIPOLAR) {
		if ((adc_code & 0x8000) == 0x8000) {	// Adc code is < 0
			adc_code = (adc_code ^ 0xFFFF) + (1 << (16 - _NUMBER_BITS));   // Convert ADC code from two's complement to binary
			sign = -1;
		}
		adc_code = adc_code >> (16 - _NUMBER_BITS);	// Shift out zero bits (2 for 14-bit, 4 for 12-bit)
		voltage = sign*(float)adc_code;
		voltage = voltage / (pow(2, _NUMBER_BITS - 1) - 1);
	}
	else {
		adc_code = adc_code >> (16 - _NUMBER_BITS);
		voltage = (float)adc_code;
		voltage = voltage / (pow(2, _NUMBER_BITS) - 1);
	}

	switch (_lastRange) {
	case Range::RANGE5V:
		voltage = voltage * 5;
		break;
	case Range::RANGE10V:
		voltage = voltage * 10;
		break;
	default:
		break;
	}

	_lastPolarity = uni_bipolar;
	_lastRange = range;

	return voltage;
}

uint8_t ExtendedADCShield::buildCommand(uint8_t channel, SignalType sgl_diff, Polarity uni_bipolar, Range range)
{
	uint8_t command = 0;

	// Build command
	switch (channel) {
	case 1:
		command = command | B01000000;
		break;
	case 2:
		command = command | B00010000;
		break;
	case 3:
		command = command | B01010000;
		break;
	case 4:
		command = command | B00100000;
		break;
	case 5:
		command = command | B01100000;
		break;
	case 6:
		command = command | B00110000;
		break;
	case 7:
		command = command | B01110000;
		break;
	default:
		break;
	}

	if (sgl_diff == SignalType::SINGLE_ENDED) {
		command = command | B10000000;
	}

	if (uni_bipolar == Polarity::UNIPOLAR) {
		command = command | B00001000;
	}

	if (range == Range::RANGE10V) {
		command = command | B00000100;
	}

	return command;
}
uint16_t ExtendedADCShield::sendSetupGetData(uint8_t command)
{
	uint16_t conv_result = 0;

	// Trigger a conversion
	digitalWrite(_CONVST, HIGH);
	digitalWrite(_CONVST, LOW);
	// Wait for BUSY to go high 
	while (digitalRead(_BUSY) == LOW);
	xSemaphoreTake(_SPImut, portMAX_DELAY);
	SPI.beginTransaction(_ADC_SPIsettings);
	digitalWrite(_RD, LOW);

	// Send command, get high byte of last conversion
	conv_result = SPI.transfer(command, SPI_CONTINUE) << 8;

	// Send filler, get low byte of last conversion
	conv_result = conv_result | SPI.transfer(B00000000);

	digitalWrite(_RD, HIGH);
	SPI.endTransaction();
	xSemaphoreGive(_SPImut);

	// Return last conversion in raw word form
	return conv_result;
}


