// ExtendedADCShield.h

#ifndef _EXTENDEDADCSHIELD_h
#define _EXTENDEDADCSHIELD_h

#if defined(ARDUINO) && ARDUINO >= 100
	#include "arduino.h"
#else
	#include "WProgram.h"
#endif

#include "FreeRTOSConfig.h" // Local config for the project, must be included before <FreeRTOS_ARM.h>.
#include <FreeRTOS_ARM.h>
#include <SPI.h>

enum class SignalType : uint8_t {
	SINGLE_ENDED = 0,
	DIFFERENTIAL = 1
}; 
enum class Polarity : uint8_t {
	UNIPOLAR = 0,
	BIPOLAR = 1
};
enum class Range : uint8_t {
	RANGE5V = 0,
	RANGE10V = 1
};

// SPI frequency
#define ADC_SPI_FREQ 8400000UL

class ExtendedADCShield
{
public:
	void init(SemaphoreHandle_t SPImut, uint8_t number_bits, uint8_t CONVST = 8, uint8_t BUSY = 9, uint8_t RD = 10);
	float analogReadConfigNext(uint8_t channel, SignalType sgl_diff, Polarity uni_bipolar, Range range);

private:
	uint8_t buildCommand(uint8_t channel, SignalType sgl_diff, Polarity uni_bipolar, Range range);
	uint16_t sendSetupGetData(uint8_t command);
	
	// Connected pins
	uint8_t _BUSY;				// Pulled low when an analog to digital conversion is in progress
	uint8_t _CONVST;			// Pull this pin high to start conversion
	uint8_t _RD;				// Read (slave select), pull low to activate SPI on ADC

	uint8_t _NUMBER_BITS;
	Polarity _lastPolarity;
	Range _lastRange;

	// Mutex for access to the SPI
	SemaphoreHandle_t _SPImut;

	// Settings for SPI communication
	SPISettings _ADC_SPIsettings;
};

#endif

