// 
// 
// 

#include "HLSerial.h"

#include "FilterParam.h"
#include "TelemetryData.h"

void HLSerialClass::Begin(uint32_t baudRate)
{
	Serial.begin(baudRate);
}

bool HLSerialClass::Read(HeaveLockMessage &input) {
	if (Serial.available()) {
		ReadState state = ReadState::LOOKING_4_HEADER;
		uint8_t dataCount = 0;
		uint8_t c;
		input.cmd = IncomingCmd::reserved_Clear;
		unsigned long startTime = millis();
		while (millis() - startTime < SERIAL_TIMEOUT) {
			if (Serial.available()) {
				c = (uint8_t)Serial.read();
			}
			else {
				continue;
			}
			
			switch (state) {
			case ReadState::LOOKING_4_HEADER:
				if (c == (uint8_t)IncomingCmd::header)
					state = ReadState::LOOKING_4_CMD;
				break;
			case ReadState::LOOKING_4_CMD:
				input.cmd = (IncomingCmd)c;
				state = ReadState::LOOKING_4_DATA_LENGTH;
				break;
			case ReadState::LOOKING_4_DATA_LENGTH:
				input.lengthOfData = c;
				if (input.lengthOfData > 0)
					state = ReadState::LOOKING_4_DATA;
				else
					state = ReadState::LOOKING_4_TAIL;
				break;
			case ReadState::LOOKING_4_DATA:
				input.bData[dataCount] = c;
				dataCount++;
				if (dataCount >= input.lengthOfData)
					state = ReadState::LOOKING_4_TAIL;
				break;
			case ReadState::LOOKING_4_TAIL:
				if (c != (uint8_t)IncomingCmd::tail) { //if it reaches this point and no tail, this means something went wrong
					this->Send(OutGoingCmd::errorGeneral, "Incomplete message received");
					return false;
				}
				else {
					return true; //done
				}
				break;
			}
		}
		this->Send(OutGoingCmd::errorGeneral, "Serial transmission timed out.");
	}
	return false;
}

float HLSerialClass::ConvertBytesToFloat(HeaveLockMessage input) {
	union Value
	{
		byte asBytes[4];
		float asFloat;
	} value;
	for (int i = 0; i < input.lengthOfData; i++) {
		value.asBytes[i] = input.bData[i];
	}
	return value.asFloat;
}

uint32_t HLSerialClass::ConvertBytesToUint32(HeaveLockMessage input) {
	union Value
	{
		byte asBytes[4];
		uint32_t asUInt;
	} value;
	for (int i = 0; i < 4; i++)
	{
		value.asBytes[i] = input.bData[i];
	}
	return value.asUInt;
}

void HLSerialClass::Send(FilterParam filtpar) {
	union Value
	{
		struct {
			byte asBytes[16];
			bool resetBuffer;
		};
		FilterParam par;
	} value;
	value.par = filtpar;
	this->serialStart(OutGoingCmd::filtPhaseParams_All, 16);
	Serial.write(value.asBytes, (size_t)16);
	this->serialEnd();
}

void HLSerialClass::Send(TelemetryData data) {
	union Value
	{
		byte asBytes[12];
		TelemetryData _data;
	} value;
	value._data = data;
	serialStart(OutGoingCmd::valPos_valSet_estVel, 12);
	Serial.write(value.asBytes, (size_t)12);
	serialEnd();
}

void HLSerialClass::Send(OutGoingCmd cmd) {
	this->serialStart(cmd, 0);
	this->serialEnd();

}

void HLSerialClass::Send(OutGoingCmd cmd, char* s) {
	uint8_t size = strlen(s);
	this->serialStart(cmd, size);
	Serial.write(s);
	this->serialEnd();
}

void HLSerialClass::Send(OutGoingCmd cmd, float f) {
	uint8_t* b = (uint8_t*)&f;
	this->serialStart(cmd, 4);
	Serial.write(b, (size_t)4);
	this->serialEnd();
}

void HLSerialClass::serialStart(OutGoingCmd cmd, uint8_t lengthOfData) {
	Serial.write((uint8_t)OutGoingCmd::header);
	Serial.write((uint8_t)cmd);
	Serial.write(lengthOfData);
}

void HLSerialClass::serialEnd() {
	Serial.write((uint8_t)OutGoingCmd::tail);
}

HLSerialClass HLSerial;