// HLSerial.h

#ifndef _HLSERIAL_h
#define _HLSERIAL_h

#if defined(ARDUINO) && ARDUINO >= 100
	#include "arduino.h"
#else
	#include "WProgram.h"
#endif

#define MAX_SERIAL_MSG_LENGTH 255
#define SERIAL_TIMEOUT 5UL

// Forward declaration:
struct FilterParam;
struct TelemetryData;

enum class OutGoingCmd : uint8_t
{
	header = 0xFE,
	tail = 0xFD,

	stopHeaveCompConfirmed = 0x02,
	startHeaveCompConfirmed = 0x03,

	statusMessage = 0x04,
	infoMessage = 0x05,
	homingStarted = 0x06,
	homingComplete = 0x07,
	homingNotExecuted = 0x08,

	valveControllerEnabled = 0x09,
	valveControllerDisabled = 0x0A,
	errorGeneral = 0x10,
	valveOpened = 0x11,
	valveClosed = 0x12,
	valveSetpointChanged = 0x14,

	valPos_valSet_estVel = 0x21,
	t2maxSetConfirmed = 0x27,
	pumpRateSetConfirmed = 0x28,
	filtPhaseParams_All = 0x29,
	MaxMotorAccelerationChanged = 0x34,

	errorAckConfirmed = 0x77
};

enum class IncomingCmd : uint8_t
{
	reserved_Clear = 0x00,
	header = 0xFE,
	tail = 0xFD,

	stopHeaveComp = 0x02,
	startHeaveComp = 0x03,
	startHoming = 0x06,

	enableValveController = 0x09,
	disableValveController = 0x0A,
	openValve = 0x11,
	closeValve = 0x12,
	setValveSetPoint = 0x14,
	getStatus = 0x15,
	get_valPos_valSet_estVel = 0x21,
	setSPfromSimulink = 0x25,
	setT2max = 0x27,
	setPumpRate = 0x28,
	setAllFiltPhaseParams = 0x29,
	setMaxMotorAcceleration = 0x34,

	errorAck = 0x77
};

struct HeaveLockMessage {
	IncomingCmd cmd;
	uint8_t lengthOfData;
	uint8_t bData[MAX_SERIAL_MSG_LENGTH];
};

class HLSerialClass
{
private:
	enum class ReadState {
		LOOKING_4_HEADER,
		LOOKING_4_CMD,
		LOOKING_4_DATA_LENGTH,
		LOOKING_4_DATA,
		LOOKING_4_TAIL
	};

	void serialStart(OutGoingCmd cmd, uint8_t lengthOfData);
	void serialEnd();

 public:
	 void Begin(uint32_t baudRate);
	 bool Read(HeaveLockMessage &input);	

	 void Send(FilterParam filtpar);
	 void Send(TelemetryData data);
	 void Send(OutGoingCmd cmd);
	 void Send(OutGoingCmd cmd, char * s);
	 void Send(OutGoingCmd cmd, float f);
	 float ConvertBytesToFloat(HeaveLockMessage input);
	 uint32_t ConvertBytesToUint32(HeaveLockMessage input);
};

extern HLSerialClass HLSerial;

#endif

