// TaskParam.h

#ifndef TaskParam_h
#define TaskParam_h

#if defined(ARDUINO) && ARDUINO >= 100
#include "arduino.h"
#else
#include "WProgram.h"
#endif

#include "FreeRTOSConfig.h" // Local config for the project, must be included before <FreeRTOS_ARM.h>.
#include <FreeRTOS_ARM.h>
//------------PARAMS---------------
#define SAMPLE_TIME_FFT 0.1f
#define SAMPLE_TIME_ACC 0.01f
#define DW 0.02f * 2.0f * (float)PI		// Frequency range around the dominant wave frequency, used in error calculation
#define T_2MAX 25.0f					// 2 % settling time of the heave filter
#define SIGMA2 sq(5.1*9.81*0.001)		// Variance of the accelerometer noise
#define S_NN sq(0.23*9.81*0.001)		// Power spectral density of the accelerometer noise
//---------------------------------

struct ParEstTaskParam {
	QueueHandle_t parameterQueue;
	QueueHandle_t bufferQueue;
	QueueHandle_t t2maxSettlingTimeQueue;
	const float sampleTimeFFT = SAMPLE_TIME_FFT;
	const float sampleTimeAcc = SAMPLE_TIME_ACC;
	const float dw = DW;
	const float T_2max = T_2MAX;
	const float sigma2 = SIGMA2;
	const float S_nn = S_NN;
};

struct MainProgramTaskParam {
	QueueHandle_t parameterQueue;
	QueueHandle_t bufferQueue;
	QueueHandle_t estVelocityQueue;
	QueueHandle_t accDataQueue;
	QueueHandle_t watchDogQueue;
	const float sampleTimeFFT = SAMPLE_TIME_FFT;
	const float sampleTimeAcc = SAMPLE_TIME_ACC;
};

struct ControllerTaskParam {
	QueueHandle_t parameterQueue;
	QueueHandle_t estVelocityQueue;
	QueueHandle_t t2maxSettlingTimeQueue;
	SemaphoreHandle_t SPImut;
};

struct ReadAccTaskParam {
	SemaphoreHandle_t SPImut;
	QueueHandle_t accDataQueue;
};


#endif

