﻿using System;
using System.Drawing;
using System.IO.Ports;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using HLC.HeaveLockCommunication;
using HLC.SimulinkCommunication;
using System.Net;
using System.Net.Sockets;

namespace HLC
{
    public partial class Form1 : Form
    {
        delegate void SetTextCallback(string text, TextBox txtBox);
        delegate void SetTextBtnCallback(string text, Button txtBox);
        delegate void ClearTextBoxCallback(TextBox txtBox);
        delegate void AppendTextCallback(string text, TextBox txtBox);
        delegate void SetLabelCallback(string text, Label label);
        delegate void SetEnableCallback(bool enable, Control c);
        delegate void CheckCheckBoxCallback(bool check, CheckBox chc);

        private HLComm myHLComm;
        private SimulinkComm mySimulinkComm;
        private Thread serialThread;

        public event EventHandler<SimulinkMessageEventArgs> valuesToSimulinkReceivedEvent;

        private void OnRaiseValuesToSimulinkReceivedEvent(SimulinkMessageEventArgs e)
        {
            // Make a temporary copy - avoid racecondition
            EventHandler<SimulinkCommunication.SimulinkMessageEventArgs> handler = valuesToSimulinkReceivedEvent;
            // Event will be null if there are no subscribers
            if (handler != null)
            {
                handler(this, e);
            }
        }

        public Form1()
        {
            InitializeComponent();
            EnableAllButtons(false);
            EnableAllCheckboxes(false);
            EnableAllNumericUpDown(false);
            SetControlEnabled(true, btnConnectSerial);
            SetControlEnabled(true, btnClearSerialPrintWind);
            picbxHeaveRunning.BackColor = Color.Yellow;
            picbxValveActive.BackColor = Color.Yellow;
            picbxHomingComplete.BackColor = Color.Yellow;
            LoadValuesToComboBox();
            mySimulinkComm = new SimulinkComm(this);
        }     

        #region HeaveLock message data
 
        void HandleSerialMessageEvent(object sender, HeaveLockMessageEventArgs e)
        {
            CheckProtocol(e.messageData);
        }

        private void CheckProtocol(HeaveLockMessage HLmsg)
        {
            switch (HLmsg.cmd)
            {
                case IncomingCmd.startHeaveCompConfirmed:
                    picbxHeaveRunning.BackColor = Color.Green;
                    SetTextButton("Stop heave compensation", btnStartStopHL);                  
                    AppendTextTextBox("Heave compensation active", txtSerialPrint);
                    SetControlEnabled(false, btnOpenValve);
                    SetControlEnabled(false, btnCloseValve);
                    SetControlEnabled(false, btnSetNewHeaveFiltParams);
                    SetControlEnabled(false, btnSetPoint);
                    SetControlEnabled(false, btnEnableValve);
                    SetControlEnabled(false, btnHomingRun);
                    SetControlEnabled(false, btnSetMaxValveAcceleration);
                    SetControlEnabled(false, chcGetSetPointFromSimulink);
                    CheckCheckBox(false, chcGetSetPointFromSimulink);
                    break;

                case IncomingCmd.stopHeaveCompConfirmed:
                    picbxHeaveRunning.BackColor = Color.Red;
                    SetTextButton("Start heave compensation", btnStartStopHL);
                    AppendTextTextBox("Heave compensation inactive", txtSerialPrint);
                    SetControlEnabled(true, btnOpenValve);
                    SetControlEnabled(true, btnCloseValve);
                    SetControlEnabled(true, btnSetNewHeaveFiltParams);
                    SetControlEnabled(true, btnSetPoint);
                    SetControlEnabled(true, btnEnableValve);
                    SetControlEnabled(true, btnHomingRun);
                    SetControlEnabled(true, btnSetMaxValveAcceleration);
                    SetControlEnabled(true, chcGetSetPointFromSimulink);
                    break;

                case IncomingCmd.statusMessage:
                    AppendTextTextBox(ByteToASCII(HLmsg.data), txtSerialPrint);
                    break;       
                            
                case IncomingCmd.infoMessage:
                    AppendTextTextBox(ByteToASCII(HLmsg.data), txtSerialPrint);
                    break;

                case IncomingCmd.valveOpened:
                    AppendTextTextBox("Valve opened", txtSerialPrint);
                    break;

                case IncomingCmd.valveClosed:
                    AppendTextTextBox("Valve closed", txtSerialPrint);
                    break;

                case IncomingCmd.valPos_valSet_estVel:
                    float  val_Pos, val_Set, est_Vel;
                    try
                    {
                        val_Pos = BitConverter.ToSingle(HLmsg.data, 0);
                        val_Set = BitConverter.ToSingle(HLmsg.data, 4);
                        est_Vel = BitConverter.ToSingle(HLmsg.data, 8);
                        SetTextTextBox(val_Pos.ToString("F3"), txtValvePos);
                        SetTextTextBox(val_Set.ToString("F3"), txtValveSetPoint);
                        SetTextTextBox(est_Vel.ToString("F3"), txtEstimatedVelocity);
                        
                        SimulinkMessage simMsg = new SimulinkMessage(HLmsg.data);
                        if (!chcFwdValPosToSim.Checked)
                            for(int i = 0; i < 4; i++)
                                simMsg.data[i] = 0x00;
                        if (!chcFwdValSetToSim.Checked)
                            for (int i = 4; i < 8; i++)
                                simMsg.data[i] = 0x00;
                        if (!chcFwdEstVelToSim.Checked)
                            for (int i = 8; i < 12; i++)
                                simMsg.data[i] = 0x00;
                        if(mySimulinkComm.IsAlive)    
                            OnRaiseValuesToSimulinkReceivedEvent(new SimulinkMessageEventArgs(simMsg));
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message);
                    }
                    break;               
                case IncomingCmd.filtPhaseParams_All:
                    if (HLmsg.data.Length == 16)
                    {
                        float a1, a0, b0, w_c;
                        a0 = BitConverter.ToSingle(HLmsg.data, 0);
                        a1 = BitConverter.ToSingle(HLmsg.data, 4);
                        b0 = BitConverter.ToSingle(HLmsg.data, 8);
                        w_c = BitConverter.ToSingle(HLmsg.data, 12);
                        SetTextTextBox(a0.ToString("F3"), txtHeaveParamA0);
                        SetTextTextBox(a1.ToString("F3"), txtHeaveParamA1);
                        SetTextTextBox(b0.ToString("F3"), txtHeaveParamB0);
                        SetTextTextBox(w_c.ToString("F3"), txtHeaveParamOmegaC);
                        SetTextTextBox(DateTime.Now.ToString("HH.mm.ss"), txtHeaveFilterParamLastTimeOfArrival);
                    }
                    else
                    {
                        AppendTextTextBox("To few bytes sendt to decode into four params", txtSerialPrint);
                    }
                    break;
                case IncomingCmd.valveSetpointChanged:
                    AppendTextTextBox("Set point changed", txtSerialPrint);
                    break;
                case IncomingCmd.homingStarted:
                    picbxHomingComplete.BackColor = Color.Yellow;
                    AppendTextTextBox("Homing started...", txtSerialPrint);
                    break;
                case IncomingCmd.homingComplete:
                    picbxHomingComplete.BackColor = Color.Green;
                    AppendTextTextBox("Homing complete", txtSerialPrint);
                    break;
                case IncomingCmd.homingNotExecuted:
                    picbxHomingComplete.BackColor = Color.Red;
                    AppendTextTextBox("The valve requires homing before operation", txtSerialPrint);
                    break;
                case IncomingCmd.valveControllerEnabled:
                    picbxValveActive.BackColor = Color.Green;
                    AppendTextTextBox("Valve controller enabled", txtSerialPrint);
                    SetTextButton("Disable valve controller", btnEnableValve);
                    SetControlEnabled(true, btnCloseValve);
                    SetControlEnabled(true, btnOpenValve);
                    SetControlEnabled(true, btnStartStopHL);
                    break;
                case IncomingCmd.valveControllerDisabled:
                    picbxValveActive.BackColor = Color.Red;
                    AppendTextTextBox("Valve controller disabled", txtSerialPrint);
                    SetTextButton("Enable valve controller", btnEnableValve);
                    SetControlEnabled(false, btnCloseValve);
                    SetControlEnabled(false, btnOpenValve);
                    SetControlEnabled(false, btnStartStopHL);
                    break;
                case IncomingCmd.errorGeneral:
                    picbxErrorWarningLight.BackColor = Color.Yellow;
                    AppendTextTextBox(ByteToASCII(HLmsg.data), txtErrorPrint);
                    break;

                case IncomingCmd.errorAckConfirmed:
                    picbxErrorWarningLight.BackColor = default(Color);
                    ClearTextTextBox(txtErrorPrint);
                    AppendTextTextBox("Error Acknowledged", txtSerialPrint);
                    break;
                case IncomingCmd.MaxMotorAccelerationChangedConfirmed:
                    AppendTextTextBox("Max valve acceleration changed", txtSerialPrint);
                    break;
                case IncomingCmd.t2maxSetConfirmed:
                    AppendTextTextBox("Max filter settling time set", txtSerialPrint);
                    break;
                case IncomingCmd.pumpRateSetConfirmed:
                    float pumpRate;
                    pumpRate = BitConverter.ToSingle(HLmsg.data, 0);
                    AppendTextTextBox("Pump rate set", txtSerialPrint);
                    SetTextTextBox(pumpRate.ToString(), txtSetPumpRate);
                    break;            
            }
        }

        private string ByteToASCII(byte b)
        {
            byte[] temp = { b };
            return Encoding.ASCII.GetString(temp);
        }

        private string ByteToASCII(byte[] b)
        {
            return Encoding.ASCII.GetString(b);
        }
        #endregion

        #region Buttons
        private void btnStartStopHL_Click(object sender, EventArgs e)
        {
            if (btnStartStopHL.Text == "Start heave compensation")
                myHLComm.SerialSend(OutGoingCmd.startHeaveComp);
            else
                myHLComm.SerialSend(OutGoingCmd.stopHeaveComp);
        }

        private void btnClearSerialPrintWind_Click(object sender, EventArgs e)
        {
            txtSerialPrint.Clear();
        }

        private void btnAckError_Click(object sender, EventArgs e)
        {
            myHLComm.SerialSend(OutGoingCmd.errorAck);
        }

        private void btnConnectSerial_Click(object sender, EventArgs e)
        {            
            if (btnConnectSerial.Text != "Disconnect")
            {
                try
                {
                    myHLComm = new HLComm(Convert.ToInt32(cmbBaudRate.SelectedItem.ToString()), cmbComPort.SelectedItem.ToString());
                    myHLComm.serialMessageReceivedEvent += HandleSerialMessageEvent;

                    serialThread = new Thread(myHLComm.ReadSerial);
                    serialThread.Name = "Serial thread";
                    serialThread.SetApartmentState(ApartmentState.STA);
                    
                    serialThread.IsBackground = true;
                    serialThread.Start();
                    while (!serialThread.IsAlive) ; //wait for thread to spawn
                    Thread.Sleep(1); //yield main to allow for some work to be done
                    btnConnectSerial.Text = "Disconnect";
                    SetLabelText("Serial open", lblSerialConnected);
                    EnableAllButtons(true);
                    EnableAllCheckboxes(true);
                    EnableAllNumericUpDown(true);
                    SetControlEnabled(false, btnStartStopHL);
                    SetControlEnabled(false, btnCloseValve);
                    SetControlEnabled(false, btnOpenValve);
                    timerGetStatus.Enabled = true;
                    timerGetStatus.Start();
                    timerGetStatus.Interval = 250;
                    picbxHeaveRunning.BackColor = Color.Yellow;
                    picbxValveActive.BackColor = Color.Yellow;
                    picbxHomingComplete.BackColor = Color.Yellow;
                    myHLComm.SerialSend(OutGoingCmd.getStatus);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
            else
            {
                try
                {
                    if (serialThread.IsAlive)
                    {
                        myHLComm.stopRead();
                    }
                    if (btnSendToSimulink.Text != "Start forwarding data to Simulink")
                    {
                        btnSendToSimulink.PerformClick();
                    }
                    if (chcGetSetPointFromSimulink.Checked) {
                        CheckCheckBox(false, chcGetSetPointFromSimulink);
                    }
                    btnConnectSerial.Text = "Connect";
                    lblSerialConnected.Text = "Serial closed";
                    timerGetStatus.Stop();
                    timerGetStatus.Enabled = false;
                    EnableAllButtons(false);
                    SetControlEnabled(true, btnClearSerialPrintWind);
                    SetControlEnabled(true, btnConnectSerial);
                    EnableAllCheckboxes(false);
                    EnableAllNumericUpDown(false);
                    ClearTextTextBox(txtValvePos);
                    ClearTextTextBox(txtValveSetPoint);
                    ClearTextTextBox(txtEstimatedVelocity);
                    picbxHeaveRunning.BackColor = Color.Yellow;
                    picbxValveActive.BackColor = Color.Yellow;
                    picbxHomingComplete.BackColor = Color.Yellow;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void btnSendToSimulink_Click(object sender, EventArgs e)
        {
            if (btnSendToSimulink.Text == "Start forwarding data to Simulink")
            {
                try
                {
                    IPEndPoint newEndPoint = new IPEndPoint(IPAddress.Parse(txtSimulinkIP.Text), int.Parse(txtSimulinkPort.Text));
                    mySimulinkComm.startSimulinkConn(newEndPoint);

                    btnSendToSimulink.Text = "Stop forwarding data to Simulink";
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
            else
            {
                try
                {
                    SimulinkMessage simMsg_fullOfZeros = new SimulinkMessage();                                       
                    OnRaiseValuesToSimulinkReceivedEvent(new SimulinkMessageEventArgs(simMsg_fullOfZeros)); //avoids hanging values in simulink
                    mySimulinkComm.stopSimulinkConn();
                    btnSendToSimulink.Text = "Start forwarding data to Simulink";
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }

        }
        private void btnOpenValve_Click(object sender, EventArgs e)
        {
            myHLComm.SerialSend(OutGoingCmd.openValve);
        }

        private void btnCloseValve_Click(object sender, EventArgs e)
        {
            myHLComm.SerialSend(OutGoingCmd.closeValve);
        }
        
        private void btnHomingRun_Click(object sender, EventArgs e)
        {
            myHLComm.SerialSend(OutGoingCmd.startHoming);
        }

        private void btnSetPoint_Click(object sender, EventArgs e)
        {
            float sp;
            try
            {
                if (!float.TryParse(txtSetPoint.Text, out sp)) { throw new Exception("Error converting setpoint to value"); }
                myHLComm.SerialSend(OutGoingCmd.setValveSetPoint, sp);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void txtSetPoint_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                btnSetPoint.PerformClick();
                txtSetPoint.SelectAll();
                e.SuppressKeyPress = true;
            }
        }

        private void btnSetNewHeaveFiltParams_Click(object sender, EventArgs e)
        {
            float a0, a1, b0, w_c;
            try
            {
                if (!float.TryParse(txtSetHeaveParamOmega_c.Text, out w_c)) { throw new Exception("Invalid w_c entry"); }               
                if (!float.TryParse(txtSetHeaveParamB0.Text, out b0)) { throw new Exception("Invalid b0 entry"); }
                if (!float.TryParse(txtSetHeaveParamA0.Text, out a0)) { throw new Exception("Invalid a0 entry"); }
                if (!float.TryParse(txtSetHeaveParamA1.Text, out a1)) { throw new Exception("Invalid a1 entry"); }
                myHLComm.SerialSend(OutGoingCmd.setAllFiltPhaseParams, new float[] { a0, a1, b0, w_c });
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btnEnableValve_Click(object sender, EventArgs e)
        {
            if (btnEnableValve.Text == "Enable valve controller")
                myHLComm.SerialSend(OutGoingCmd.enableValveController);
            else
                myHLComm.SerialSend(OutGoingCmd.disableValveController);
        }

        private void btnGetLocalIP_Click(object sender, EventArgs e)
        {
            IPHostEntry host;
            string localIP = "?";
            host = Dns.GetHostEntry(Dns.GetHostName());
            foreach (IPAddress ip in host.AddressList)
            {
                if (ip.AddressFamily == AddressFamily.InterNetwork)
                {
                    localIP = ip.ToString();
                }
            }
            txtSimulinkIP.Text = localIP;
        }

        private void btnSetMaxValveAcceleration_Click(object sender, EventArgs e)
        {
            try
            {
                uint maxAcc;
                if (!uint.TryParse(txtMaxAcceleration.Text, out maxAcc)) { throw new Exception("Invalid max acceleration entry"); }
                if (maxAcc > 0)
                    myHLComm.SerialSend(OutGoingCmd.setMaxMotorAcceleration, maxAcc);
                else
                    MessageBox.Show("Acceleration must be a positive integer");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void txtMaxAcceleration_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                btnSetMaxValveAcceleration.PerformClick();
                txtMaxAcceleration.SelectAll();
                e.SuppressKeyPress = true;
            }
        }

        private void btnSetPumpRate_Click(object sender, EventArgs e)
        {
            float pumpRate;
            try
            {
                if (!float.TryParse(txtSetPumpRate.Text, out pumpRate)) { throw new Exception("Invalid pump rate entry"); }
                
                myHLComm.SerialSend(OutGoingCmd.setPumpRate, pumpRate);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void txtSetPumpRate_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                btnSetPumpRate.PerformClick();
                txtSetPumpRate.SelectAll();
                e.SuppressKeyPress = true;
            }
        }

        private void btnSetT2Max_Click(object sender, EventArgs e)
        {
            float t2max;
            try
            {
                if (!float.TryParse(txtSetT2max.Text, out t2max)) { throw new Exception("Invalid settling time entry"); }
                if (t2max < 0) { throw new Exception("Negative values not allowed"); }
                myHLComm.SerialSend(OutGoingCmd.setT2max, t2max);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void txtSetT2max_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                btnSetT2Max.PerformClick();
                txtSetT2max.SelectAll();
                e.SuppressKeyPress = true;
            }
        }

        #endregion

        #region ComboBoxes
        private void cmbComPort_MouseClick(object sender, MouseEventArgs e)
        {
            try
            {
                cmbComPort.DataSource = SerialPort.GetPortNames();
                cmbComPort.SelectedIndex = 0;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error loading serial ports. Try restarting HeaveLock Interface. Error: " + ex.Message);
            }
        }
        
        private void cmbComPort_MouseDown(object sender, MouseEventArgs e)
        {
            try
            {
                cmbComPort.DataSource = SerialPort.GetPortNames();
                cmbComPort.SelectedIndex = 0;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error loading serial ports. Try restarting HeaveLock Interface. Error: " + ex.Message);
            }
        }
        private void cmbBaudRate_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (btnConnectSerial.Text != "Connect")
                myHLComm.setBaudRate(Convert.ToInt32(cmbBaudRate.SelectedItem.ToString()));
        }
        private void LoadValuesToComboBox()
        {
            cmbBaudRate.Items.Add(9600);
            cmbBaudRate.Items.Add(115200);
            cmbBaudRate.Items.Add(230400);
            cmbBaudRate.Items.Add(238636); 
            cmbBaudRate.Items.Add(250000);
            cmbBaudRate.SelectedIndex = 3;
            try
            {
                this.cmbComPort.Items.Clear();
                cmbComPort.DataSource = SerialPort.GetPortNames();
                cmbComPort.SelectedIndex = cmbComPort.Items.Count - 1;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error loading serial ports. Try restarting HeaveLock Interface. Error: ", ex.Message);
            }
        }        
        #endregion

        #region Edit controls and form
        private void SetTextTextBox(string text, TextBox txtBox)
        {
            if (txtBox.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextTextBox);
                this.Invoke(d, new object[] { text, txtBox });
            }
            else
            {
                txtBox.Text = text;
            }
        }
        private void ClearTextTextBox( TextBox txtBox)
        {
            if (txtBox.InvokeRequired)
            {
                ClearTextBoxCallback d = new ClearTextBoxCallback(ClearTextTextBox);
                this.Invoke(d, new object[] { txtBox });
            }
            else
            {
                txtBox.Clear();
            }
        }
        private void SetTextButton(string text, Button btn)
        {
            if (btn.InvokeRequired)
            {
                SetTextBtnCallback d = new SetTextBtnCallback(SetTextButton);
                this.Invoke(d, new object[] { text, btn });
            }
            else
            {
                btn.Text = text;
            }
        }
        private void AppendTextTextBox(string text, TextBox txtBox)
        {
            if (txtBox.InvokeRequired)
            {
                AppendTextCallback d = new AppendTextCallback(AppendTextTextBox);
                this.Invoke(d, new object[] { text, txtBox });
            }
            else
            {
                txtBox.AppendText(DateTime.Now.ToString("HH.mm.ss") + " - " + text + "\r\n");
            }
        }
        private void SetLabelText(string text, Label label)
        {
            if (label.InvokeRequired)
            {
                SetLabelCallback d = new SetLabelCallback(SetLabelText);
                this.Invoke(d, new object[] { text, label });
            }
            else
            {
                label.Text = text;
            }
        }

        private void SetControlEnabled(bool enable, Control c)
        {
            if (c.InvokeRequired)
            {
                SetEnableCallback d = new SetEnableCallback(SetControlEnabled);
                this.Invoke(d, new object[] { enable, c });
            }
            else
            {
                c.Enabled = enable;
            }
        }

        private void CheckCheckBox(bool check, CheckBox chc)
        {
            if (chc.InvokeRequired)
            {
                CheckCheckBoxCallback d = new CheckCheckBoxCallback(CheckCheckBox);
                this.Invoke(d, new object[] { check, chc });
            }
            else
            {
                chc.Checked = check;
            }
        }

        private void EnableAllButtons(bool enable)
        {
            foreach (Control c in this.Controls)
            {
                if (c is GroupBox)
                {
                    foreach (Control cc in c.Controls)
                    {
                        if (cc is Button)
                        {
                            SetControlEnabled(enable, cc);
                        }
                    }
                }
            }
            foreach(TabPage t in tbCntrlSetUpAndManual.TabPages)
            {
                foreach(Control c in t.Controls)
                {
                    if(c is Button)
                        SetControlEnabled(enable, c);
                }
            }
        }
        
        private void EnableAllCheckboxes(bool enable)
        {
            foreach (Control c in this.Controls)
            {
                if (c is GroupBox)
                {
                    foreach (Control cc in c.Controls)
                    {
                        if (cc is CheckBox)
                        {
                            SetControlEnabled(enable, cc);
                        }
                    }
                }
            }
            foreach (TabPage t in tbCntrlSetUpAndManual.TabPages)
            {
                foreach (Control c in t.Controls)
                {
                    if (c is CheckBox)
                        SetControlEnabled(enable, c);
                    if (c is GroupBox)
                    {
                        foreach (Control cc in c.Controls)
                        {
                            if (cc is CheckBox)
                                SetControlEnabled(enable, cc);
                        }
                    }
                }
            }
        }     
               
        private void EnableAllNumericUpDown(bool enable)
        {
            foreach (Control c in this.Controls)
            {
                if (c is GroupBox)
                {
                    foreach (Control cc in c.Controls)
                    {
                        if (cc is NumericUpDown)
                        {
                            SetControlEnabled(enable, cc);
                        }
                    }
                }
            }
            foreach (TabPage t in tbCntrlSetUpAndManual.TabPages)
            {
                foreach (Control c in t.Controls)
                {
                    if (c is NumericUpDown)
                        SetControlEnabled(enable, c);
                }
            }
        }
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            DialogResult result = MessageBox.Show("Are you sure you want to exit HeaveLock Control?", "Exit?", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (!result.Equals(DialogResult.Yes))
            {
                e.Cancel = true;
            }
            else
            {
                try
                {
                    if (serialThread != null && serialThread.IsAlive)
                    {
                        myHLComm.stopRead();
                    }
                    if (btnSendToSimulink.Text != "Start forwarding data to Simulink")
                    {
                        btnSendToSimulink.PerformClick();
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error trying to exit serial communication: ", ex.Message, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }       
        }

        #endregion

        #region Timer
        private void timerGetStatus_Tick(object sender, EventArgs e)
        {
            if (picbxValveActive.BackColor == Color.Yellow || picbxHeaveRunning.BackColor == Color.Yellow)
                myHLComm.SerialSend(OutGoingCmd.getStatus);
        }
        #endregion
          
        #region Temporary function for receiving UDP data
        UdpClient udpServer;

        public void UDPDataReceived(IAsyncResult ar)
        {
            UdpClient client = ((UdpState)(ar.AsyncState)).client;
            if (chcGetSetPointFromSimulink.Checked)
            {
                try
                {
                    IPEndPoint wantedIpEndPoint = ((UdpState)(ar.AsyncState)).ep;
                    IPEndPoint receivedIpEndPoint = new IPEndPoint(IPAddress.Any, 0);
                    Byte[] receiveBytes = client.EndReceive(ar, ref receivedIpEndPoint);
                    
                    if (((wantedIpEndPoint.Address.Equals(receivedIpEndPoint.Address)) || (wantedIpEndPoint.Address.Equals(IPAddress.Any))))
                    {
                        double convertedData2 = BitConverter.ToDouble(receiveBytes, 0);
                        myHLComm.SerialSend(OutGoingCmd.setSPfromSimulink, Convert.ToSingle(convertedData2));
                       

                    }
                    client.BeginReceive(new AsyncCallback(UDPDataReceived), ar.AsyncState);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Error UDPDataReceived", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                client.Close();
            }
        }

        private void chcGetSetPointFromSimulink_CheckedChanged(object sender, EventArgs e)
        {
            if (chcGetSetPointFromSimulink.CheckState == CheckState.Checked)
            {               
                try
                {
                    int listenPort = 57777;
                    IPEndPoint localEP = new IPEndPoint(IPAddress.Any, listenPort);
                    udpServer = new UdpClient(localEP);
                    IPEndPoint remoteEP = new IPEndPoint(IPAddress.Parse(txtSimulinkIP.Text), 0);
                    UdpState state = new UdpState();
                    state.client = udpServer;
                    state.ep = remoteEP;
                    udpServer.BeginReceive(new AsyncCallback(UDPDataReceived), state);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }                
            }
            else
            {
                udpServer.Close();
                            
            }
        }        
        #endregion
    }
}