﻿namespace HLC.HeaveLockCommunication
{
    /// <summary>
    /// IncomingMsgTypeCodes is a static class that contains predefined message code values.
    /// These values consist of one byte each, and can not be set to be 0xFF or 0x00 (used for error). 
    /// </summary>
    public static class IncomingCmd
    {
        private const byte reserved1                    = 0xFF;
        private const byte reserved2                    = 0x00;

        public const byte header                        = 0xFE;
        public const byte tail                          = 0xFD;
        
        public const byte stopHeaveCompConfirmed        = 0x02;
        public const byte startHeaveCompConfirmed       = 0x03;

       
        public const byte statusMessage                 = 0x04;
        public const byte infoMessage                   = 0x05;
        public const byte homingStarted                 = 0x06;
        public const byte homingComplete                = 0x07;
        public const byte homingNotExecuted             = 0x08;
        public const byte valveControllerEnabled        = 0x09;
        public const byte valveControllerDisabled       = 0x0A;
        public const byte errorGeneral                  = 0x10;

        public const byte valveOpened                   = 0x11;
        public const byte valveClosed                   = 0x12;   
        public const byte valveSetpointChanged          = 0x14;

        public const byte valPos_valSet_estVel          = 0x21;
        public const byte t2maxSetConfirmed             = 0x27;
        public const byte pumpRateSetConfirmed          = 0x28;
        public const byte filtPhaseParams_All           = 0x29;
        public const byte MaxMotorAccelerationChangedConfirmed   = 0x34;


        public const byte errorAckConfirmed             = 0x77;

        /// <summary>
        /// ValidateCmd checks that the code supplied as a parameter exists
        /// </summary>
        /// <param name="code"></param>
        /// <returns>True if exists, false if not</returns>
        public static bool ValidateCmd(byte code)
        {
            System.Reflection.FieldInfo[] fields = typeof(IncomingCmd).GetFields();
            foreach (System.Reflection.FieldInfo f in fields)
            {
                if ((byte)f.GetValue(null) == code)
                    return true;

            }
            return false;
        }
    }
}
