﻿namespace HLC.HeaveLockCommunication
{
    /// <summary>
    /// OutGoingMsgTypeCodes is a static class that contains predefined message code values.
    /// These values consist of one byte each, and can not be set to be 0xFF or 0x00 (used for error). 
    /// </summary>
    public static class OutGoingCmd
    {
        private const byte reserved1                = 0xFF;
        private const byte reserved2                = 0x00;

        public const byte header                    = 0xFE;
        public const byte tail                      = 0xFD;
                
        public const byte stopHeaveComp             = 0x02;
        public const byte startHeaveComp            = 0x03;
        public const byte startHoming               = 0x06;

        public const byte enableValveController     = 0x09;
        public const byte disableValveController    = 0x0A;
        
        public const byte openValve                 = 0x11;
        public const byte closeValve                = 0x12;
        public const byte setValveSetPoint          = 0x14;
        public const byte getStatus                 = 0x15;

        public const byte get_valPos_valSet_estVel  = 0x21;
        
        public const byte setSPfromSimulink         = 0x25;
        public const byte setT2max                  = 0x27;
        public const byte setPumpRate           = 0x28;
        public const byte setAllFiltPhaseParams     = 0x29;
        public const byte setMaxMotorAcceleration   = 0x34;

        public const byte errorAck					= 0x77;

        /// <summary>
        /// ValidateCmd checks that the code supplied as a parameter exists
        /// </summary>
        /// <param name="code"></param>
        /// <returns>True if exists, false if not</returns>
        public static bool ValidateCmd(byte code)
        {
            System.Reflection.FieldInfo[] fields = typeof(OutGoingCmd).GetFields();

            foreach (System.Reflection.FieldInfo f in fields)
            {
                if ((byte)f.GetValue(null) == code)
                    return true;
            }
            return false;
        }

    }
}
