﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Windows.Forms;

namespace HLC.SimulinkCommunication
{
    public class SimulinkComm
    {
        private volatile bool _keepUDPAlive = false;

        private int _localSendingPort;
        private volatile UdpClient _udpClient;
        private IPEndPoint _remoteEP;

        public SimulinkComm(Form1 form1)
        {
            form1.valuesToSimulinkReceivedEvent += HandleUDPMessageEvent;
        }

        private void HandleUDPMessageEvent(object sender, SimulinkMessageEventArgs e)
        {
            if (_keepUDPAlive)
            {
                UDPsend(e.messageData);
            }
        }

        private void UDPsend(SimulinkMessage msg)
        {
                try
                {                    
                    _udpClient.Send(msg.data, msg.Length, _remoteEP);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.ToString());
                }
        }

        public void stopSimulinkConn()
        {
            _keepUDPAlive = false;
            _udpClient.Close();
        }

        public void startSimulinkConn(IPEndPoint remoteEP)
        {
            _localSendingPort = 57780;
            _remoteEP = remoteEP;
            _udpClient = new UdpClient(_localSendingPort);
            _keepUDPAlive = true;
        }

        public bool IsAlive { get { return _keepUDPAlive; } }

    }
}
